import re
from collections.abc import Iterable


def re_extract_try(_object, _pattern, group=0):
    if isinstance(_object, str):
        try:
            return re.search("%s" % _pattern, _object).group(group)
        except AttributeError:
            return None
    if isinstance(_object, int):
        try:
            return re.search("%s" % _pattern, str(_object)).group(group)
        except AttributeError:
            return None


def re_find_all(_object, _pattern, loc=None):
    if isinstance(_object, str):
        try:
            if loc:
                return re.findall("%s" % _pattern, _object)[loc]
            else:
                return re.findall("%s" % _pattern, _object)
        except AttributeError:
            return None
    if isinstance(_object, int):
        try:
            if loc:
                return re.findall("%s" % _pattern, str(_object))[loc]
            else:
                return re.findall("%s" % _pattern, str(_object))
        except AttributeError:
            return None


def re_extract(_object, _pattern, group=0):
    if isinstance(_object, str):
        if isinstance(_pattern, str):
            try:
                if __local_extractor(_object, _pattern, group) is not None:
                    yield __local_extractor(_object, _pattern, group)
                else:
                    return None
            except AttributeError:
                return None
        elif isinstance(_pattern, Iterable):
            for sub in _pattern:
                yield from re_extract(_object, sub)
        else:
            return None
    elif isinstance(_object, Iterable):
        for thing in _object:
            if isinstance(_pattern, str):
                yield from re_extract(thing, _pattern)
            elif isinstance(_pattern, Iterable):
                for sub in _pattern:
                    yield from re_extract(thing, sub)
            else:
                return None
    else:
        return None


def __local_extractor(string_to_search, pattern_to_search, group):
    m_find = re.search("%s" % pattern_to_search, string_to_search)
    if m_find:
        return m_find.group(group)
    else:
        return None


# TODO: Make this maintainable
def re_search_strings():
    search_strings = {
        "search_string_RLS": r"(RLS-)(.+)(?<=\d)(?=_)",
        "search_string_full_data_RLS": r"(RLS-)(.+)(?<=\d)(?=_)",
        "search_string_results_form": r"(AI_RLS-)(.+)(?<=\d)(?=_)",
        "search_string_RLS_directory": r"(RLS.+)(\\)",
        "search_string_doc_file": r"^(RLS)(.+)(doc)",
        "search_string_spectrum_word": r"(Spectrum)(.+)(doc)",
        "search_string_spectrum_csv": r"(Spectrum)(.+)(csv)",
        "search_string_LIMS": r"^(19)(.+?)(?=_)|(?<=_)(19\d{4,})",
        "search_string_date_first": r"^(2019)(\d+)",
        "search_string_date": r"(2019)(\d+)",
        "search_string_test": r"(47)(.+?)(?=_)",
        "search_string_free_text": r"(?<=_)([a-zA-Z].+)",
        "search_string_match_all_after_last_underscore": r"([^_]*)$",
        "search_string_excel_lims": r"(\w.+)(?=19\d{5})(19\d{5})",
        "search_string_excel_spectrum": r"^(19\d{5}|20\d{5})(=? )(\w.+)",
        "search_string_match_exactly_seven_digits": r"(?:\b\d{7}\b)",
        "search_string_match_any_words": r"([a-zA-z])",
        "search_string_match_AI_lims_first": r"(19\d{5}|20\d{5}).([\w]*.+[^\d\W])",
        "search_string_match_AI_Name_First": r"([\w]*.+[^\d\W]).(19\d{5}|20\d{5})",
        "search_string_match_AI": r"(([\w]*.+)(\b..\b)(19\d{5}|20\d{5}))|(("
        r"19\d{5}|20\d{5}).([\w]*.+[^\d\W]))|(([\w]*.+[^\d\W])(\b.\b)(19\d{5}|20\d{5}))|((19\d{5}|20\d{5})(\b.\b)([\w]*.+[^\d\W]))",
        "seach_string_LIMS_in_files_Future": r"(19\d{5}|20\d{5})(?=_)",
        "search_string_AI_match_word": r"([^\d\W]+)",
        "search_string_AI_numeric_only": r"^([\d]+)",
        # Starting From Here We Can Maintain A Bit Easier
        "lims_bidirectional": r"(\b\d{7}\b)",
        "lims_image_files": r"(JPG)|(PPTX)|(PNG)",
    }
    return search_strings


def regex_yield(_object, _pattern):
    if _object:
        if isinstance(_object, str):
            if isinstance(_pattern, str):
                if re.search(_pattern, _object):
                    yield _object
                else:
                    return None
            elif not isinstance(_pattern, str):
                for sub_pattern in _pattern:
                    yield from regex_yield(_object, sub_pattern)
        elif not isinstance(_object, str):
            for thing in _object:
                if isinstance(_pattern, str):
                    if re.search(_pattern, thing):
                        yield [thing]
                elif not isinstance(_pattern, str):
                    for sub_pattern in _pattern:
                        yield from regex_yield(thing, sub_pattern)
    else:
        return None



from tkinter import *
from tkinter import filedialog
from tools import lazy_lims_new, reshape_spectral_csv


class CheckBar(Frame):
    def __init__(self, parent=None, picks=None, side=LEFT, anchor=W):
        Frame.__init__(self, parent)
        if picks is None:
            picks = {}
        self.vars = []
        for pick in picks.keys():
            var = IntVar()
            chk = Checkbutton(master=self, text=pick, variable=var)
            chk.pack(side=side, anchor=anchor, expand=YES)
            self.vars.append((var, picks[pick]))

    def state(self):
        return filter(
            None, map(lambda var: var[1] if var[0].get() else None, self.vars)
        )


class Browse(Frame):
    def __init__(self, parent=None, label=None, anchor=S, side=LEFT):
        Frame.__init__(self, parent)
        self.directory = ""
        if not label:
            self.label = Label(parent, font=("Helvetica", 9), fg="red")
        self.btn = Button(
            parent, text="Browse", command=lambda: self.browsefunc()
        ).pack(anchor=anchor, side=side)

    def browsefunc(self):
        self.directory = filedialog.askdirectory()
        self.label.config(text=self.directory.split("/")[-1])
        self.label.pack()

    def get(self):
        return self.directory


class OkButton(Frame):
    def __init__(
        self,
        parent=None,
        label=None,
        anchor=S,
        side=RIGHT,
        function=None,
        *args,
        **kwargs
    ):
        Frame.__init__(self, parent)
        self.ok = Button(
            parent, text="Ok", command=lambda: function(*args, **kwargs)
        ).pack(anchor=anchor, side=side)
        self.label = label


def tools_ui():
    root = Tk()
    functions_name = {
        "Lims and Sample Name": lazy_lims_new,
        "EDS Convert": reshape_spectral_csv,
    }
    lng = CheckBar(root, functions_name)
    lng.pack(side=TOP, fill=X)
    br = Browse(root)
    br.pack()
    ok = OkButton(root, function=c, lng=lng, directory=br, root=root)
    ok.pack()
    root.mainloop()


def c(lng, directory, root):
    directory = directory.get()
    for function in list(lng.state()):
        function(directory)
    root.destroy()


if __name__ == "__main__":
    tools_ui()


import glob
import os

from numpy import nan
import pandas as pd

from .retools import *


def set_nested_dictionary(d_, root, one, two=None):
    m_two = set(two) if two else set()

    try:
        d_[root][one].add(two)
    except KeyError:
        try:
            d_[root].update({one: m_two})
        except KeyError:
            d_[root] = {one: m_two}
        except TypeError:
            d_[root].update({one: set()})


def deep(*args, d=None):
    root = args[0]
    if not d:
        d = {}
    if isinstance(args, tuple):
        args = [data for data in args]
    for arg in args[1:]:
        d[root] = {arg, deep(args[1:], d)}
    return d


def is_dir(lazy_path):
    path_list = [
        globed
        for globed in glob.glob(r"{}".format(lazy_path + "/*"))
        if os.path.isdir(globed)
    ]
    return path_list


def tools_generate_used_techniques_from_raw(path):
    technique_lims = {}
    techniques = {"FTIR", "SEM", "EDS", "OptDigMicr", "mFTIR", "TG"}
    bad_optical = {r"(OptDig)(.+)?(?=_)", r"(Optic)(.+)?(?=_)"}

    m_search_strings = re_search_strings()

    for current_path, dir_in_cur_path, files_in_cur_path, in os.walk(path):
        for directory in dir_in_cur_path:
            used_techniques = [data for data in re_extract(directory, techniques)]
            bad_tech = [data for data in re_extract(directory, bad_optical)]
            for bad in bad_tech:
                # TODO: Refactor this side effect
                temp = directory.replace(
                    re.search(bad, directory).group(0), "OptDigMicr"
                )
                new_directory = os.path.join(path, temp)
                old_directory = os.path.join(path, directory)
                used_techniques = ["OptDigMicr"]
                try:
                    os.rename(old_directory, new_directory)
                except FileNotFoundError:
                    pass
            files = glob.glob(os.path.join(current_path, directory + "\\*"))
            for file in files:
                m_lims = re_extract_try(
                    file.split("\\")[-1],
                    m_search_strings["seach_string_LIMS_in_files_Future"],
                    0,
                )
                if m_lims:
                    try:
                        set_nested_dictionary(
                            technique_lims, used_techniques[0], m_lims, "x"
                        )
                    except:
                        pass

    return technique_lims


def lazy_lims_new(root):
    from collections import defaultdict

    path_list = is_dir(root)
    lims_dict = defaultdict()
    techniques = {"FTIR", "SEM", "EDS", "OptDigMicr", "mFTIR", "TG"}
    for path in path_list:
        for current_path, dir_in_cur_path, files_in_cur_path, in os.walk(path):
            for file in files_in_cur_path:
                try:
                    date, lims, name = os.path.splitext(file)[0].split("_")
                    tech = [d for d in re_extract(current_path, techniques)][0]
                    old_name = lims_dict.get(lims, {}).get("Name")
                    if lims_dict.get(lims):
                        lims_dict[lims]["Name"] = (
                            old_name
                            if (old_name and len(old_name) < len(name))
                            else name
                        )
                        lims_dict[lims][tech] = "x"
                    else:
                        lims_dict[lims] = {"Name": name, tech: "x"}
                except ValueError as e:
                    print(e, file)
                    continue

    t = pd.DataFrame.from_dict(lims_dict).transpose().reset_index()
    t["Lims_Name"] = t[["Name", "index"]].apply(lambda x: " ".join(x), axis=1)
    t = t.drop(columns=["Name"])
    t.set_index("index", inplace=True)
    t.sort_index(inplace=True)
    ordered_tech = ["Lims_Name", "OptDigMicr", "FTIR", "SEM", "EDS"]
    for tech in ordered_tech:
        if tech not in t:
            t[tech] = ""
    t = t[ordered_tech]
    t.index.name = "LIMS"
    t.to_csv(root + "\\lims.csv", sep=";")


def lazy_lims(lazy_path):
    m_search_strings = re_search_strings()
    path_list = is_dir(lazy_path)
    lims_sample_container = {}
    lims_index = []
    tech = tools_generate_used_techniques_from_raw(lazy_path)

    for path in path_list:
        for current_path, dir_in_cur_path, files_in_cur_path, in os.walk(path):
            for file in files_in_cur_path:
                if not re_extract_try(
                    file, m_search_strings["search_string_spectrum_csv"]
                ):
                    m_free_text = re_extract_try(
                        file,
                        m_search_strings[
                            "search_string_match_all_after_last_underscore"
                        ],
                        0,
                    )
                    if m_free_text:
                        m_free_text = m_free_text.split(".")[0]
                    m_lims = re_extract_try(
                        file, m_search_strings["seach_string_LIMS_in_files_Future"], 0
                    )
                    if m_lims:
                        lims_index.append(m_lims)
                    if m_lims and m_free_text:
                        if lims_sample_container.get(m_lims):
                            lims_sample_container[m_lims].add(m_free_text)
                        else:
                            lims_sample_container[m_lims] = {m_free_text}

    # Logically the shortest name should be the correct sample name
    # This fails when there are multiple samples attached to a given lims
    for k, v in lims_sample_container.items():
        lims_sample_container[k] = min(filter(None, v), key=len)
    lims_index_set = set(lims_index)
    lims_index = list(lims_index_set)
    lims_index.sort()
    dls = ["{0} {1}".format(v, k) for k, v in lims_sample_container.items()]

    # Ensures we're sorting by LIMS #
    dls = sorted(dls, key=lambda x: x.split(" ")[-1])
    lazy_data = pd.DataFrame(index=lims_index, data=dls, columns=["LIMS"])
    ordered_tech = ["OptDigMicr", "FTIR", "SEM", "EDS"]

    for order in ordered_tech:
        lazy_data[order] = nan
        for k, v in tech.items():
            if k == order:
                for kk, vv in v.items():
                    lazy_data.loc[kk, k] = next(iter(vv))

    lazy_data.fillna("na", inplace=True)
    lazy_data.to_csv(lazy_path + "\\lims and sample.csv", sep=";")

    lims_frame = pd.DataFrame(data=lims_index)
    lims_frame.to_csv(lazy_path + "\\lims.csv", sep=";")


def reshape_spectral_csv(lazy_path):
    elemental_dictionary = {
        "Ni": "Nickel (%)",
        "Si": "Silicon (%)",
        "Mg": "Magnesium (%)",
        "Al": "Aluminum (%)",
        "K": "Potassium (%)",
        "Ca": "Calcium (%)",
        "Cu": "Copper (%)",
        "Zr": "Zirconium (%)",
        "Mo": "Molybdenum (%)",
        "Ag": "Silver (%)",
        "Au": "Gold (%)",
        "Sn": "Tin (%)",
        "W": "Tungsten (%)",
        "Cr": "Chromium (%)",
        "Fe": "Iron (%)",
        "S": "Sulfur (%)",
        "Cl": "Chlorine (%)",
        "Zn": "Zinc (%)",
        "P": "Phosphor (%)",
        "Pb": "Lead (%)",
        "Ne": "Neon (%)",
        "Ti": "Titanium(%)",
        "Mn": "Manganese (%)",
    }

    path_list = is_dir(lazy_path)
    for p in path_list:
        if re.search("SEM", p) or re.search("EDS", p):
            try:
                spectra = glob.glob(p + "\\*.csv")
                eds_frame = pd.read_csv(spectra[0], skiprows=2, index_col="Name")
                eds_frame.rename(columns=elemental_dictionary, inplace=True)
                hold = []
                for i in eds_frame.values:
                    temp_hold = []
                    for index, value, in enumerate(i):
                        value = str(value)
                        if value not in " ":
                            temp_hold.append(
                                "{} {}".format(value, eds_frame.columns[index])
                            )
                    hold.append(temp_hold)
                e = eds_frame.reset_index()
                cleaned = pd.DataFrame(
                    data=hold,
                    index=e["Name"].apply(
                        lambda x: " ".join("_".join(x.split("_")[::-1]).split("_")[:-1])
                    ),
                )
                with open(lazy_path + "\\cleaned_spectrum.csv", "w") as lzy:
                    cleaned.to_csv(
                        lzy, sep=";", header=True, index=True, line_terminator="\n"
                    )
            except Exception as e:
                print(e)
                pass
